package com.ekexiu.console.system.service;

import com.ekexiu.console.system.dao.OrgDao;
import com.ekexiu.console.system.dao.OrgUserDao;
import com.ekexiu.console.system.po.OrgUser;
import com.ekexiu.console.system.po.Organization;
import com.ekexiu.console.system.pojo.OrganizationInfo;
import com.ekexiu.console.system.vo.ConsoleAuthUser;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.DefaultValue;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.param.Like;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Delete;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.*;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.io.IoUtil;

import java.io.*;
import java.sql.Connection;
import java.sql.SQLException;

@Path("/sys/org")
public class OrgService extends com.ekexiu.console.service.Upload {

    private File imagePath;
    private File tmpPath;
    private File orgPath;
    public static final String DEFAULT_PW_STR = StringUtil.md5("12345678");

    public File getImagePath() {
        return imagePath;
    }

    public void setImagePath(File imagePath) {
        this.imagePath = imagePath;
        this.orgPath = new File(this.imagePath, "org");
    }

    public File getTmpPath() {
        return tmpPath;
    }

    public void setTmpPath(File tmpPath) {
        this.tmpPath = tmpPath;
    }

    public File getOrgPath() {
        return orgPath;
    }

    public void setOrgPath(File orgPath) {
        this.orgPath = orgPath;
    }

    @Autowrie
    private OrgDao orgDao;
    @Autowrie
    private OrgUserDao orgUserDao;

    public OrgUserDao getOrgUserDao() {
        return orgUserDao;
    }

    public void setOrgUserDao(OrgUserDao orgUserDao) {
        this.orgUserDao = orgUserDao;
    }

    public OrgDao getOrgDao() {
        return orgDao;
    }

    public void setOrgDao(OrgDao orgDao) {
        this.orgDao = orgDao;
    }

    @Post
    @Path
    public String insert(@JdbcConn(true) Connection con, @RequestParam(excludeFields = {"createTime", "modifyTime", "id"}) Organization orgn)
            throws SQLException {
        String id = StringUtil.buildUUID();
        orgn.setId(id);
        this.orgDao.insert(con, orgn);
        return id;
    }

    public void saveOrgLogo(String id, String fn) throws IOException {
        InputStream in = new FileInputStream(new File(this.tmpPath, fn));
        try {
            IoUtil.copy(in, new FileOutputStream(new File(this.orgPath, id + ".jpg")), false, true);
        } finally {
            in.close();
        }
    }

    public boolean hasOrgLogo(String id) {
        String orgPath = this.orgPath+"/"+id+".jpg";
        File file = new File(orgPath);
        if(file.exists()){
            return true;
        }else{
            return false;
        }
    }

    @Post
    @Path("/create")
    public String insertOrg(@JdbcConn(true) Connection con,
                            @RequestParam(fields = {@FieldParam(value = "email", valueClass = String.class, required = true),
                                    @FieldParam(value = "inviterId", valueClass = String.class, required = true),
                                    @FieldParam(value = "name", valueClass = String.class, required = true),
                                    @FieldParam(value = "forShort", valueClass = String.class, required = true),
                                    @FieldParam(value = "province", valueClass = String.class, required = true),
                                    @FieldParam(value = "city", valueClass = String.class, required = true),
                                    @FieldParam(value = "foundTime", valueClass = String.class, required = false),
                                    @FieldParam(value = "descp", valueClass = String.class, required = false),
                                    @FieldParam(value = "orgType", valueClass = String.class, required = false),
                                    @FieldParam(value = "industry", valueClass = String.class, required = false),
                                    @FieldParam(value = "subject", valueClass = String.class, required = false),
                                    @FieldParam(value = "orgSize", valueClass = String.class, required = false),
                                    @FieldParam(value = "qualification", valueClass = String.class, required = false),
                                    @FieldParam(value = "orgUrl", valueClass = String.class, required = false)}) OrganizationInfo info,@Nullable String fn)
            throws SQLException,IOException {
        String id = StringUtil.buildUUID();
        OrgUser orgUser = new OrgUser();
        info.setId(id);
        orgUser.setId(id);
        orgUser.setEmail(info.getEmail());
        orgUser.setPasswd(DEFAULT_PW_STR);
        orgUser.setInviterId(info.getInviterId());
        orgUser.setUserType("1");
        if(fn != null){
            this.saveOrgLogo(orgUser.getId(), fn);
        }
        this.orgUserDao.insert(con, orgUser);
        this.orgDao.insert(con, info);
        return orgUser.getId();
    }

    @Post
    @Path("/update")
    public void update(@JdbcConn(true) Connection con, Organization orgn,@Nullable String fn) throws SQLException, IOException {
        if(fn != null){
            this.saveOrgLogo(orgn.getId(), fn);
        }
        this.orgDao.update(con, orgn);
    }

    @Get
    @Path("/pq")
    public PageQueryResult<OrganizationInfo> pageQuery(@JdbcConn Connection con, @LoginUser ConsoleAuthUser cau, @Nullable @Like String name, @Nullable @Like String industry, @Nullable @Like String subject, @Nullable @Like String city, @DefaultValue("1") int pageNo, @DefaultValue("10") int pageSize) throws SQLException {
        String cName = null;
        if (name != null) {
            cName = "%" + name + "%";
        }
        String cIndustry = null;
        if (industry != null) {
            cIndustry = "%" + industry + "%";
        }
        String cSubject = null;
        if (subject != null) {
            cSubject = "%" + subject + "%";
        }
        String cCity = null;
        if (city != null) {
            cCity = "%" + city + "%";
        }
        return this.orgDao.query(con,cau.getId(), cName, cIndustry, cSubject, cCity, pageSize, pageNo);
    }

    @Get
    @Path("/id/{id}")
    public OrganizationInfo query(@JdbcConn Connection con, @PathVar String id) throws SQLException {
        OrganizationInfo organizationInfo = this.orgDao.query(con, id);
        if (organizationInfo != null) {
            organizationInfo.setHasOrgLogo(this.hasOrgLogo(organizationInfo.getId()));
        }
        return organizationInfo;
    }

    @Delete
    @Path("/id/{id}")
    public void deleteOne(@JdbcConn(true) Connection con, @PathVar String id) throws SQLException {
        this.orgDao.delete(con, id);
    }


    @Post
    @Path("/del")
    public void delete(@JdbcConn(true) Connection con, String[] ids) throws SQLException {
        this.orgDao.delete(con, ids);
    }


}
